import openai
import pinecone
import os

# Set the OpenAI API key
openai.api_key = os.environ["OPENAI"]
# set up pinecone
pinecone.init(api_key=os.environ["PINECONE"], environment="us-west4-gcp-free")

query = (
    "what safety measures should be taken in shopping malls to prevent fire hazards?"
)
# Generate text embeddings using OpenAI

res = openai.Embedding.create(input=[query], engine="text-embedding-ada-002")


# retrieve from pinecone
index = pinecone.Index("mojave")

xq = res["data"][0]["embedding"]

# get relevant context
res = index.query(xq, top_k=5, include_metadata=True)
# print(res)

# contexts

contexts = [item["metadata"]["text"] for item in res["matches"]]

# print(contexts)

augmented_query = "\n\n---\n\n".join(contexts) + "\n\n---\n\n" + query

# print(augmented_query)



import openai
import os
import pinecone
from tqdm.auto import tqdm
import datetime
from time import sleep
import loadTest
from loadTest import chunks

# text = loadTest.chunks
# print(input)
# Set the OpenAI API key
openai.api_key = os.environ["OPENAI"]

embed_model = "text-embedding-ada-002"

# res = openai.Embedding.create(input=text, engine=embed_model)

# print(res.keys())
# print(len(res["data"]))

# initilize pinecone

index_name = "mojave"
dimension = 1536

# initialize connection
pinecone.init(api_key=os.environ["PINECONE"], environment="us-west4-gcp-free")

# check if index exists
if index_name not in pinecone.list_indexes():
    pinecone.create_index(index_name, dimension=dimension, metric="dotproduct")

# connect to index
index = pinecone.Index(index_name)
# view index stat
# print(index.describe_index_stats())

# upsert embeddings
chunks = loadTest.chunks
# print(chunks)
# print(len(chunks))

batch_size = 100

# ...
# your previous code

# initialize an empty list to accumulate embeddings and metadata
all_embeddings = []
all_meta_batch = []

for i in tqdm(range(0, len(chunks), batch_size)):
    # find the end of the batch
    i_end = min(len(chunks), i + batch_size)
    meta_batch = chunks[i:i_end]
    # print(meta_batch)
    # get ids
    ids_batch = [x["id"] for x in meta_batch]
    # print(len(ids_batch))
    # get texts to encode
    texts = [x["text"] for x in meta_batch]
    # print(len(texts))
    # create embeddings (try-except to avoid RateLimitError)

    try:
        res = openai.Embedding.create(input=texts, engine=embed_model)
        print(len(res["data"][0]["embedding"]))
    except:
        done = False
        while not done:
            sleep(5)
            try:
                res = openai.Embedding.create(input=texts, engine=embed_model)
                print(len(res["data"][0]["embedding"]))
                done = True
            except:
                pass

    # accumulate embeddings and metadata
    embeds = [record["embedding"] for record in res["data"]]
    all_embeddings.extend(embeds)
    all_meta_batch.extend(meta_batch)

# clean up metadata
to_upsert = [
    (
        record["id"],
        embedding,
        {"text": record["text"], "chunk": record["chunk"], "file_name": record["file"]},
    )
    for record, embedding in zip(all_meta_batch, all_embeddings)
]

# upsert embeddings
index.upsert(vectors=to_upsert)


# for i in tqdm(range(0, len(chunks), batch_size)):
#     # find the end of the batch
#     i_end = min(len(chunks), i + batch_size)
#     meta_batch = chunks[i:i_end]
#     print(meta_batch)
#     # get ids
#     ids_batch = [x["id"] for x in meta_batch]
#     # print(len(ids_batch))
#     # get texts to encode
#     texts = [x["text"] for x in meta_batch]
#     # print(len(texts))
#     # create embeddings (try-except to avoid RateLimitError)
#     try:
#         res = openai.Embedding.create(input=texts, engine=embed_model)
#     except:
#         done = False
#         while not done:
#             sleep(5)
#             try:
#                 res = openai.Embedding.create(input=texts, engine=embed_model)
#                 done = True
#             except:
#                 pass
# embeds = [record["embedding"] for record in res["data"]]
# # clean up metadata
# meta_batch = [
#     {"text": x["text"], "chunk": x["chunk"], "file_name": x["file"]} for x in meta_batch
# ]
# to_upsert = list(zip(ids_batch, embeds, meta_batch))
# # upsert embeddings
# index.upsert(vectors=to_upsert)



# There are a few things we need to do go really understand the logic of the system.
# The human specifies what their learning goal is and how long they have to achieve it.
# The system then uses the goal to determine a schedule of subject for the human to learn in order to achive that goal.

# Lets start with the human specifying their learning goal.
# The human specifies their learning goal by answering the following questions:
# 1. What is your learning goal?
# 2. How long do you have to achieve your learning goal?
# 3. What is your current level of knowledge of the subject?

#  an input statmemt that asks the user to specify their learning goal

goal = input("What is your learning goal?")
time = input("How long do you have to achieve your learning goal?")

# Now er append the goal and time to the augmented query.

human_statement = f"""My learning goal is{goal}and I have{time}to achieve it."""
print(human_statement)

# Now we have to pass the human input into the model to the system.

# we need to import the openai library and the os library

import openai
import os

# initialize the openai

# Set the OpenAI API key
openai.api_key = os.environ["OPENAI"]

# system message to prime the model
primer = f"""You are Q&A bot. A highly intelligent system that answers human questions based on the information provded by the user above each question.
If the information can not be found in the information provided by the user, come up with a plausible answer.
"""

res = openai.ChatCompletion.create(
    model="gpt-3.5-turbo-16k-0613",
    messages=[
        {"role": "system", "content": primer},
        {"role": "user", "content": human_statement},
    ],
    temperature=1,
    max_tokens=256,
    top_p=1,
    frequency_penalty=0,
    presence_penalty=0,
)

# first we have to get the system to first perform a mental simuation of the human's learning goal.
# using this mental model the system determines the skills needed to function and solve problems in the subject area.
# using these skills, it goes on to determine the subject set that the human needs to learn in order to achieve their learning goal.

# first we write a function for the mental model that takes the human statement as input and returns the mental model as output.


def mental_model(human_statement):
    # logic for the mental model
    skills = []
    # first we define what it means to achieve the human's goal.
    # next we try to figure out what kinds of problems the human will have to solve in order to be that person.
    # `skills` is a list of skills that the human will need to solve the problems.
    # we then use the skills to determine the subject set that the human needs to learn in order to achieve their learning goal.
    # foe example if the human wants to be a doctor, we define the problem a doctor is tasked to solve as they diagnose and treat patients.
    # the skills needed to do this are:
    # 1. the ability to diagnose patients
    # 2. the ability to treat patients
    # 3. the ability to prescribe medication
    # 4. the ability to perform surgery
    # 5. the ability to perform medical procedures
    # 6. the ability to perform medical tests
    # among other things.
    # the subject set that the human needs to learn in order to achieve their learning goal is:
    # 1. anatomy
    # 2. physiology
    # 3. biochemistry
    # 4. pharmacology
    # 5. pathology
    # 6. microbiology
    # 7. immunology
    # 8. genetics
    # 9. epidemiology
    # 10. medical ethics
    # 11. medical law
    # 12. medical terminology
    # among other things.
    # so what if my goal is to be a fashion desigher?



import openai
import os
import read
from read import augmented_query

augmented_query = read.augmented_query
# print(augmented_query)

# Set the OpenAI API key
openai.api_key = os.environ["OPENAI"]

# system message to prime the model
primer = f"""You are Q&A bot. A highly intelligent system that answers human questions based on the information provded by the user above each question.
If the information can not be found in the information provided by the user, come up with a plausible answer.
"""

res = openai.ChatCompletion.create(
    model="gpt-3.5-turbo-16k-0613",
    messages=[
        {"role": "system", "content": primer},
        {"role": "user", "content": augmented_query},
    ],
    temperature=1,
    max_tokens=256,
    top_p=1,
    frequency_penalty=0,
    presence_penalty=0,
)

print(res["choices"][0]["message"]["content"])
# response = display(Markdown(res["choices"][0]["message"]["content"]))
# print(response)



import subprocess
import sys

commit_types = {
    "build:": "major",
    "feat:": "minor",
    "fix:": "patch",
}

try:
    # Get the commit message from command-line arguments
    commit_message = sys.argv[1]
    if not commit_message:
        raise ValueError("Please provide a commit message.")

    # Determine the type of change based on the commit message
    commit_type = next(
        (prefix for prefix in commit_types.keys() if commit_message.startswith(prefix)),
        None,
    )

    # Read the current version from pyproject.toml
    with open("pyproject.toml", "r") as toml_file:
        toml_content = toml_file.read()
        current_version = toml_content.split('version = "')[1].split('"')[0]

    if commit_type:
        # Increment the version based on the commit type
        parts = current_version.split(".")
        version_bump = commit_types[commit_type]
        if version_bump == "major":
            parts[0] = str(int(parts[0]) + 1)
            parts[1] = "0"
            parts[2] = "0"
        elif version_bump == "minor":
            parts[1] = str(int(parts[1]) + 1)
            parts[2] = "0"
        else:
            parts[2] = str(int(parts[2]) + 1)
        next_version = ".".join(parts)
    else:
        print("No recognized commit type found. Using patch version bump.")
        parts = current_version.split(".")
        parts[2] = str(int(parts[2]) + 1)
        next_version = ".".join(parts)

    # Update pyproject.toml with the new version
    toml_content = toml_content.replace(
        f'version = "{current_version}"', f'version = "{next_version}"'
    )
    with open("pyproject.toml", "w") as toml_file:
        toml_file.write(toml_content)

    # Commit the version update and tag the commit
    subprocess.run(["git", "add", "."], check=True)
    subprocess.run(["git", "commit", "-m", commit_message], check=True)

    # Create the appropriate tag
    tag_message = (
        f"{commit_type.capitalize()} release: {next_version}"
        if commit_type
        else f"Patch release: {next_version}"
    )
    subprocess.run(
        ["git", "tag", f"v{next_version}", "-a", "-m", tag_message], check=True
    )
    subprocess.run(["git", "push"], check=True)
    subprocess.run(["git", "push", "origin", f"v{next_version}"], check=True)

    # # Build distribution archives
    # subprocess.run(["python", "-m", "pip", "install", "--upgrade", "build"], check=True)
    # subprocess.run(["python", "-m", "build"], check=True)

    # # Upload the distribution archives to PyPI using twine
    # subprocess.run(["python", "-m", "pip", "install", "--upgrade", "twine"], check=True)
    # subprocess.run(
    #     ["python", "-m", "twine", "upload", "--repository", "testpypi", "dist/*"],
    #     check=True,
    # )
except Exception as e:
    print("An error occurred:", str(e))



import os
from PyPDF2 import PdfReader

#  Get the path of the current file
currentPath = os.path.abspath(__file__)

# Get the directory containing the current file

currentDir = os.path.dirname(currentPath)

#  construct the path to the nba facts folder

folderPath = os.path.join(currentDir, "instructions")

print(folderPath)


def extract(folderPath):
    # create a list of all the files
    pdfFiles = []
    # create a loop to go through all the in the folder path
    for file in os.listdir(folderPath):
        # add a condition to read only files with .pdf extensions
        if file.endswith(".pdf"):
            pdfFiles.append(os.path.join(file))
    return pdfFiles


pdfFiles = extract(folderPath)

# print(pdfFiles)


def read(pdf_files):
    # create a dictionary that stores key value pairs of the title of the file and the content of the file
    datasets = {}

    for file in pdf_files:
        try:
            # open the PDF file using PdfReader
            pdf_reader = PdfReader(file)

            # Now we exatract the text from each page of the file
            text = ""
            for page_num in range(len(pdf_reader.pages)):
                page = pdf_reader.pages[page_num]
                text += page.extract_text()

            # store extracted text in the dictionary
            file_name = os.path.basename(file)
            datasets[file_name] = text

        except Exception as e:
            print(f"Error processing {file}: {e}")

    data = []
    for file_name, text in datasets.items():
        data.append({"file_name": file_name, "text": text})

    return data


dataset = read(pdfFiles)
print(dataset)



# define a function to get pdf files from a folder and store them in a python list with their file paths.
import os
from PyPDF2 import PdfReader
from uuid import uuid4
from tqdm.auto import tqdm
import tiktoken
from langchain.text_splitter import RecursiveCharacterTextSplitter

# Get the path of the current file
current_file_path = os.path.abspath(__file__)

# Get the directory containing the current file
current_dir_path = os.path.dirname(current_file_path)

# Construct the path to the instructions folder relative to the current file
folder_path = os.path.join(current_dir_path, "instructions")


def extract(folder_path):
    pdf_files = []
    for file in os.listdir(folder_path):
        if file.endswith(".pdf"):
            pdf_files.append(os.path.join(folder_path, file))
    return pdf_files


pdf_files = extract(folder_path)


def load(pdf_files):
    instructions = {}  # Dictionary to store file name -> extracted text pairs

    for file in pdf_files:
        try:
            # Open the PDF file using PdfReader
            pdf_reader = PdfReader(file)

            # Extract text from each page
            text = ""
            for page_num in range(len(pdf_reader.pages)):
                page = pdf_reader.pages[page_num]
                text += page.extract_text()
            # print(text)

            # Store extracted text in the dictionary
            file_name = os.path.basename(file)
            instructions[file_name] = text

        except Exception as e:
            print(f"Error processing {file}: {e}")

    data = []
    for file_name, text in instructions.items():
        data.append({"file_name": file_name, "text": text})

    return data


# Call the function and get the text dictionary
pdf_text_dict = load(pdf_files)
# print(pdf_text_dict)

# Print the dictionary
# for file_name, text in pdf_text_dict.items():
#     print(f"File: {file_name}\nText: {text}\n")
text = pdf_text_dict[0]["text"]
# print(text)
# chunking the paragraph output into sizable pieces
tokenizer = tiktoken.get_encoding("p50k_base")


# create the length function
def tiktoken_len(text):
    tokens = tokenizer.encode(text, disallowed_special=())
    return len(tokens)


text_splitter = RecursiveCharacterTextSplitter(
    chunk_size=400,
    chunk_overlap=20,
    length_function=tiktoken_len,
    separators=[" ", "\n", "\n\n", ""],
)

# print(text_splitter.split_text(text))

# # even more processing

chunks = []
for record in tqdm(pdf_text_dict):
    pdf_chunks = text_splitter.split_text(record["text"])
    chunks.extend(
        [
            {"id": str(uuid4()), "text": chunk, "chunk": i, "file": record["file_name"]}
            for i, chunk in enumerate(pdf_chunks)
        ]
    )

# print(len(chunks))
print(chunks[0])
# let's see how this adds up to everything else.



import openai
import os
from IPython.display import Markdown, display
import read
from read import augmented_query

augmented_query = read.augmented_query

# Set the OpenAI API key
openai.api_key = os.environ["OPENAI"]

# System message to prime the model
primer = f"""You are a Q&A bot. A highly intelligent system that answers human questions based on the information provided by the human above each question.
If the information cannot be found in the information provided by the user, you truthfully say "I don't know". 
"""

# Get augmented query
user_input = augmented_query  # Assuming the function returns user input

# Create chat completion
res = openai.ChatCompletion.create(
    model="gpt-4",
    messages=[
        {"role": "system", "content": primer},
        {"role": "user", "content": user_input},
    ],
)

# Get the response content
response_content = res["choices"][0]["message"]["content"]

# Display the Markdown content (for Jupyter Notebook)
# markdown_response = Markdown(response_content)
# display(markdown_response)

# Print the response content (for standard Python script)
# print(response_content)

# we need more tests for the job of mental model function. it seems like something we'll have to do with the model itself
